/*
 * QEMU4V tracer
 *
 *  Copyright (c) 2015-2018 Sergey Smolov
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see <http://www.gnu.org/licenses/>.
 */

#ifndef Q4V_TRACER_H_
#define Q4V_TRACER_H_

#include <string.h>
#include <stdbool.h>
#include <stdint.h>

void enable_q4v_trace(void);
bool q4v_trace_enabled(void);

void enable_q4v_pte(void);
bool q4v_pte_enabled(void);

int q4v_tstamp(void);
void set_q4v_tstamp(int tstamp);

void q4v_trace_banked_reg_write(const char *regname, uint64_t val, int banked);
void q4v_trace_write_regname_prefix(const char *regname);
void log_write_prefix(const int reg);
void q4v_trace_value(const uint64_t value);
void q4v_trace_reg_write(const char *regname, const uint64_t value);

void q4v_trace_mem_access(const char *rw,
    const int size, const uint64_t mem_addr, const uint64_t data);

/* Utility functions. */
char *trimwhitespace(char *str);

struct insninfo {

    /* Instruction address. */
    uint32_t insn_addr;

    /* Instruction string representation. */
    char insn[256];
    struct insninfo *next;
};

/* Filled in by disas.c - Information about instructions. */
extern struct insninfo *insninfos;

#endif /* Q4V_TRACER_H_ */
